% Black-Derman-Toy interest rate tree fitted to both
% the yield curve AND volatilities.
% Shows that the market prices of ZCB are recovered from the tree,
% and so are the yield volatilities.

clc; clear;

% Tenors, rates, volatilities, and discount factors (P).
%         1w   2w    3w    1m    3m    6m    1y  2y   3y   5y   7y  10y 20y
Tenors = [1/52 2/52  3/52  1/12  3/12  6/12  1    2    3    5    7    10   20  30];
Rates  = [2.11 2.225 2.355 2.46  2.565 3.18  4.27 5.61 6.04 7.02 8.79 9.48 9.6 9.8 ]./100;
sigma  = [ 5.3  5.2   5.1   4.9   4.7   4.0   3.6  3.2  2.6  2.5  2.1  2.0 1.8 1.5]./100;

% Time (T), number of time steps (N) and increment (dt).
% Twenty-five year bonds
T = 25;
N = 25;
dt = T/N;
Ti = [dt:dt:T+dt];
Ri = interp1(Tenors,Rates,Ti);
P = 1./(1+Ri.*dt).^Ti;
sigmaI = interp1(Tenors,sigma,Ti);

% Initialize the "up" and "down" yield vectors Pu and Pd.
Pu = zeros(1,N+1);
Pd = zeros(1,N+1);

% Find the vectors Pu and Pd of up and down yields.
options = optimset('TolFun', 1e-20);
for i=2:N+1
	Pu(i) = fminsearch(@(Pu) findPu(Pu,P(i),Ri(1),sigmaI(i),dt), 0.9, options);
	Pd(i) = Pu(i)^exp(-2*sigmaI(i)*sqrt(dt));
end

% Set the objective function 
% 'ABS' (absolute errors) or 'SSE' (squared errors)
ObjFun = 'ABS';

% Return the BDT rates (r), discount factors (d), and forward vols (fsigma)
[r d fsigma] = BuildBDTtree_yields_vols(Ri(1),sigmaI(1),Pu,Pd,N,dt,ObjFun);
r.*100

% Calculate the zero coupon bond prices from the tree.
for m=1:N
	Bond(m) = BDTDiscountBond(m,N,d);
end

% Compare the tree prices to those from the market.
% They are the same.
ZCBComparison = [Bond' P(1:end-1)']
ZCB_MSE = mean((Bond'-P(1:end-1)').^2)

% Compare the tree prices to those from the market.
% They are the same.
RecoveredSigma = -log(log(Pd(2:end))./log(Pu(2:end)))./2./sqrt(dt);
SigmaComparison = [sigmaI(2:N+1)' RecoveredSigma']
Sigma_MSE = mean((sigmaI(2:N+1)' - RecoveredSigma').^2) 

